#!/usr/bin/env python3

import csv
import argparse
import urllib.request, urllib.error, urllib.parse
import json

from intigua_client import IntiguaRestClient


def display_users():
    try:
        client = IntiguaRestClient(args.user, args.apikey, args.url)
        print("Successfully fetched Users: ")
        print(client.listUsers())
    except urllib.error.HTTPError as e:
        if "BadIp" in e.read():
            print("Failed to display_users - Bad IP, exiting...")
            exit(1)
        print("Failed to display_users. This might be due to general problem")
        print(e)
        print(e.read())
        exit(1)

parser = argparse.ArgumentParser(description='Display Users')
parser.add_argument('--url', dest='url', type=str,
                    help='URL to Intigua Server. Please use full rest path. e.g:https://[intigua-servername]/vmanage-server/rest/rest-api',
                    required=True)
parser.add_argument('--user', dest='user', type=str, help='Username', required=True)
parser.add_argument('--apikey', dest='apikey', type=str, help='API Key', required=True)
args = parser.parse_args()


def main():
    display_users()


if __name__ == '__main__':
    main()
